@@ -1037,9 +1037,7 @@ X == Y :- same_term X Y.
       !: !fresh_int)),
   DocAbove);
 
-  LPDoc  {|[findall_solution P L] finds all the solved instances of P and puts them in L
-in the order in which they are found. Instances can contain eigenvariables
-and unification variables.|};
+  LPDoc  {|[findall_solution P L] finds all the solved instances of P and puts them in L in the order in which they are found. Instances can contain eigenvariables and unification variables. P may or may not be instantiated. Instances should be found in L.|};
   LPCode "external pred findall_solutions i:prop, o:list prop.";
 
   MLData safe;
@@ -1200,6 +1198,27 @@ set,
     (fun m _ ~depth -> !: (Set.elements m))),
   DocAbove);
 
+  MLCode(Pred(name^".choose",
+    In(set,"M",
+    Out(alpha,"X",
+    Easy "X is an element of M")),
+    (fun m _ ~depth -> !: (try Set.choose m with Not_found -> raise No_clause))),
+  DocAbove);
+
+  MLCode(Pred(name^".min",
+    In(set,"M",
+    Out(alpha,"X",
+    Easy "X is the smallest element of M")),
+    (fun m _ ~depth -> !: (try Set.min_elt m with Not_found -> raise No_clause))),
+  DocAbove);
+
+  MLCode(Pred(name^".max",
+    In(set,"M",
+    Out(alpha,"X",
+    Easy "X is the bigger of M")),
+    (fun m _ ~depth -> !: (try Set.max_elt m with Not_found -> raise No_clause))),
+  DocAbove);
+
   MLCode(Pred(name^".cardinal",
     In(set,"M",
     Out(int,"N",
@@ -1233,6 +1252,35 @@ set,
     )),
   DocAbove);
 
+  MLCode(Pred(name^".fold",
+    In(set,"M",
+    In(poly "A","Acc",
+    In(HOAdaptors.pred2a alpha "A","F",
+    Out(poly "A","Acc1",
+    FullHO(ContextualConversion.unit_ctx, "fold M w.r.t. the predicate F"))))),
+    (fun m a f _ ~once ~depth _ _ state ->
+
+      let state, a, gls = HOAdaptors.fold1 ~once ~depth ~fold:Set.fold f m a state in
+      
+      state, !: a, gls
+    )),
+  DocAbove);
+
+
+  MLCode(Pred(name^".partition",
+  In(set,"M",
+  In(HOAdaptors.pred1 alpha,"F",
+  Out(set,"M1",
+  Out(set,"M2",
+  FullHO(ContextualConversion.unit_ctx, "Partitions M w.r.t. the predicate F, M1 is where F holds"))))),
+  (fun m f _ _ ~once ~depth _ _ state ->
+
+    let state, (m1, m2), gls = HOAdaptors.filter1 ~once ~depth ~filter:Set.partition f m state in
+    
+    state, !: m1 +! m2, gls
+  )),
+  DocAbove);
+
 
 ] 
 ;;
@@ -1337,6 +1385,21 @@ let open BuiltIn in let open BuiltInData in
     )),
   DocAbove);
 
+
+  MLCode(Pred(name^".fold",
+    In(map "A","M",
+    In(poly "C","Acc",
+    In(HOAdaptors.pred3a alpha closed_A "C","F",
+    Out(poly "C","Acc1",
+    FullHO(ContextualConversion.unit_ctx, "fold M w.r.t. the predicate F"))))),
+    (fun m a f _ ~once ~depth _ _ state ->
+
+      let state, a, gls = HOAdaptors.fold2 ~once ~depth ~fold:Map.fold f m a state in
+      
+      state, !: a, gls
+    )),
+  DocAbove);
+
 ] 
 ;;
 
