@@ -42,7 +42,7 @@ module Term = struct
 
 (* Used by pretty printers, to be later instantiated in module Constants *)
 let pp_const = mk_spaghetti_printer ()
-type constant = int (* De Bruijn levels *)
+type constant = int (* De Bruijn levels *) [@@ deriving ord]
 let pp_constant = pp_spaghetti pp_const
 let show_constant = show_spaghetti pp_const
 let equal_constant x y = x == y
@@ -106,7 +106,7 @@ and uvar_body = {
   mutable contents : term [@printer (pp_spaghetti_any ~id:id_term pp_oref)];
   mutable uid_private : int; (* unique name, the sign is flipped when blocks a constraint *)
 }
-[@@deriving show]
+[@@deriving show, ord]
 
 (* we use this projection to be sure we ignore the sign *)
 let uvar_id { uid_private } = abs uid_private [@@inline];;
@@ -116,7 +116,7 @@ let uvar_isnt_a_blocker { uid_private } = uid_private > 0 [@@inline];;
 let uvar_set_blocker r   = r.uid_private <- -(uvar_id r) [@@inline];;
 let uvar_unset_blocker r = r.uid_private <-  (uvar_id r) [@@inline];;
 
-type arg_mode = Util.arg_mode = Input | Output [@@deriving show]
+type arg_mode = Util.arg_mode = Input | Output [@@deriving show, ord]
 
 type clause = {
     depth : int;
@@ -128,13 +128,11 @@ type clause = {
 }
 and 
 mode = arg_mode list
-[@@deriving show]
+[@@deriving show, ord]
 
 let to_mode = function true -> Input | false -> Output
 
 
-module DT = Discrimination_tree
-
 type stuck_goal = {
   mutable blockers : blockers;
   kind : unification_def stuck_goal_kind;
@@ -172,7 +170,7 @@ and second_lvl_idx =
     mode : mode;
     arg_depths : int list;   (* the list of args on which the trie is built *)
     time : int;         (* time is used to recover the total order *)
-    args_idx : clause DT.t; 
+    args_idx : clause Discrimination_tree.t; 
 }
 [@@deriving show]
 
@@ -591,18 +589,20 @@ module CHR : sig
 
   val empty : t
 
-  val new_clique : (constant -> string) -> constant list -> t -> t * clique
-  val clique_of : constant -> t -> Constants.Set.t option
+  val new_clique : (constant -> string) -> constant list -> constant list -> t -> t * clique
+  val clique_of : constant -> t -> (Constants.Set.t * Constants.Set.t) option
   val add_rule : clique -> rule -> t -> t
   val in_clique : clique -> constant -> bool
   
   val rules_for : constant -> t -> rule list
 
   val pp : Fmt.formatter -> t -> unit
+  val pp_clique : Fmt.formatter -> clique -> unit
   val show : t -> string
 
 end = struct (* {{{ *)
 
+  type clique = {ctx_filter: Constants.Set.t; clique: Constants.Set.t} [@@deriving show]
   type sequent = { eigen : term; context : term; conclusion : term }
   and rule = {
     to_match : sequent list;
@@ -617,40 +617,51 @@ end = struct (* {{{ *)
   }
   [@@ deriving show]
   type t = {
-    cliques : Constants.Set.t Constants.Map.t;
+    cliques : clique Constants.Map.t;
     rules : rule list Constants.Map.t
   }
   [@@ deriving show]
-  type clique = Constants.Set.t
 
   let empty = { cliques = Constants.Map.empty; rules = Constants.Map.empty }
 
-  let in_clique m c = Constants.Set.mem c m
+  let in_clique {clique; ctx_filter} c = Constants.Set.mem c clique
 
-  let new_clique show_constant cl ({ cliques } as chr) =
+  let new_clique show_constant hyps cl ({ cliques } as chr) =
+    let open Constants in
     if cl = [] then error "empty clique";
-    let c = List.fold_right Constants.Set.add cl Constants.Set.empty in
-    Constants.Map.iter (fun _ c' ->
-      if not (Constants.Set.is_empty (Constants.Set.inter c c')) && not (Constants.Set.equal c c') then
-        error ("overlapping constraint cliques: {" ^
-          String.concat "," (List.map show_constant (Constants.Set.elements c))^"} {" ^
-          String.concat "," (List.map show_constant (Constants.Set.elements c'))^ "}")
-    ) cliques;
-    let cliques =
-      List.fold_right (fun x cliques -> Constants.Map.add x c cliques) cl cliques in
-    { chr with cliques }, c
+    let c = Set.of_list cl in
+    let ctx_filter = Set.of_list hyps in
+    
+    (* Check new inserted clique is valid *)
+    let build_clique_str c =
+      Printf.sprintf "{ %s }" @@ String.concat "," (List.map show_constant (Set.elements c)) 
+    in
+    let old_ctx_filter = ref None in
+    let exception Stop in
+    (try 
+      Map.iter (fun _ ({clique=c';ctx_filter=ctx_filter'}) ->
+        if Set.equal c c' then (old_ctx_filter := Some ctx_filter'; raise Stop)
+        else if not (Set.disjoint c c') then (* different, not disjoint clique *)
+          error ("overlapping constraint cliques:" ^ build_clique_str c ^ "and" ^ build_clique_str c')
+      ) cliques;
+    with Stop -> ());
+    let clique = 
+      {ctx_filter = Set.union ctx_filter (Option.value ~default:Set.empty !old_ctx_filter); clique=c} in
+    let (cliques: clique Constants.Map.t) =
+      List.fold_left (fun cliques x -> Constants.Map.add x clique cliques) cliques cl in
+    { chr with cliques }, clique
 
   let clique_of c { cliques } =
-    try Some (Constants.Map.find c cliques)
+    try Some (let res = Constants.Map.find c cliques in res.clique, res.ctx_filter)
     with Not_found -> None
 
-  let add_rule cl r ({ rules } as chr) =
+  let add_rule ({clique}: clique) r ({ rules } as chr) =
     let rules = Constants.Set.fold (fun c rules ->
       try
         let rs = Constants.Map.find c rules in
         Constants.Map.add c (rs @ [r]) rules
       with Not_found -> Constants.Map.add c [r] rules)
-      cl rules in
+      clique rules in
     { chr with rules }
 
 
@@ -671,7 +682,7 @@ type clause_w_info = {
 [@@ deriving show]
 
 type macro_declaration = (Ast.Term.t * Loc.t) F.Map.t
-[@@ deriving show]
+[@@ deriving show, ord]
 
 exception No_clause
 exception No_more_steps
