@@ -50,6 +50,10 @@ module Func = struct
 
   end
 
+  let is_uvar_name s =
+    let c = s.[0] in
+    ('A' <= c && c <= 'Z')
+
   include Self
   module Map = Map.Make(Self)
 
@@ -64,7 +68,9 @@ module Term = struct
    | CData of CData.t
    | Quoted of quote
   and quote = { data : string; loc : Loc.t; kind : string option }
-  [@@deriving show]
+  [@@deriving show, ord]
+
+exception NotInProlog of Loc.t * string
 
 let mkC x = CData x
 let mkLam x t = Lam (Func.from_string x,t)
@@ -79,7 +85,11 @@ let mkQuoted loc s =
     match s.[i] with
     | '{' -> find_data (i+1)
     | ':' ->
-       let rec find_space i = match s.[i] with
+       let len = String.length s in
+       let rec find_space i =
+         if i >= len then
+           raise (NotInProlog(loc,"syntax error: bad named quotation: {{"^s^"}}.\nDid you separate the name from the data with a space as in {{:name data}} ?."));
+         match s.[i] with
          | ' ' -> i 
          | '\n' -> i 
          | _ -> find_space (i+1) in
@@ -99,7 +109,6 @@ let mkSeq l =
  in
   aux l
 
-exception NotInProlog of Loc.t * string
 
 let rec best_effort_pp = function
  | Lam (x,t) -> "x\\" ^ best_effort_pp t
@@ -145,7 +154,7 @@ module Clause = struct
     attributes : 'attributes;
     body : 'term;
   }
-  [@@deriving show]
+  [@@deriving show, ord]
 
 end
 
@@ -160,7 +169,7 @@ module Chr = struct
     attributes : 'attribute;
     loc: Loc.t;
   }
-  [@@deriving show]
+  [@@deriving show, ord]
 
 
 
@@ -173,7 +182,7 @@ module Macro = struct
      name : 'name;
      body : 'term
   }
-  [@@deriving show]
+  [@@deriving show, ord]
 
 end
 
@@ -185,7 +194,7 @@ module Type = struct
     name : Func.t;
     ty : Term.t;
   }
-  [@@deriving show]
+  [@@deriving show, ord]
 
 end
 
@@ -193,7 +202,7 @@ module Mode = struct
 
   type 'name t =
     { name : 'name; args : bool list; loc : Loc.t }
-  [@@deriving show]
+  [@@deriving show, ord]
 
 end
 
@@ -201,7 +210,7 @@ module TypeAbbreviation = struct
 
   type ('name) t =
     { name : 'name; value : Term.t; nparams : int; loc : Loc.t }
-  [@@deriving show]
+  [@@deriving show, ord]
 
 end
 
@@ -212,7 +221,7 @@ module Program = struct
     (* Blocks *)
     | Begin of Loc.t
     | Namespace of Loc.t * Func.t
-    | Constraint of Loc.t * Func.t list
+    | Constraint of Loc.t * Func.t list * Func.t list
     | Shorten of Loc.t * (Func.t * Func.t) list (* prefix suffix *)
     | End of Loc.t
 
@@ -289,22 +298,27 @@ type program = {
   modes : Func.t Mode.t list;
   body : block list;
 }
+and cattribute = {
+  cid : string;
+  cifexpr : string option
+}
+and block_constraint = {
+   clique : Func.t list;
+   ctx_filter : Func.t list;
+   rules : cattribute Chr.t list
+}
 and block =
   | Locals of Func.t list * program
   | Clauses of (Term.t,attribute) Clause.t list
   | Namespace of Func.t * program
   | Shorten of Func.t shorthand list * program
-  | Constraints of Func.t list * cattribute Chr.t list * program
+  | Constraints of block_constraint * program
 and attribute = {
   insertion : insertion option;
   id : string option;
   ifexpr : string option;
 }
 and insertion = Before of string | After of string | Replace of string
-and cattribute = {
-  cid : string;
-  cifexpr : string option
-}
 and tattribute =
   | External
   | Index of int list * tindex option
@@ -314,6 +328,6 @@ and 'a shorthand = {
   full_name : 'a;
   short_name : 'a;
 }
-[@@deriving show]
+[@@deriving show, ord]
 
 end
