@@ -32,6 +32,8 @@ module Func : sig
   val dummyname : t
   val spillf : t
 
+  val is_uvar_name : t -> bool
+
   val from_string : string -> t
 
   module Map : Map.S with type key = t
@@ -46,7 +48,7 @@ module Term : sig
    | CData of CData.t
    | Quoted of quote
   and quote = { data : string; loc : Loc.t; kind : string option }
-  [@@ deriving show]
+  [@@ deriving show, ord]
 
   exception NotInProlog of Loc.t * string
 
@@ -82,7 +84,7 @@ module Clause : sig
     attributes : 'attributes;
     body : 'term;
   }
-  [@@ deriving show]
+  [@@ deriving show, ord]
 
 end
 
@@ -127,7 +129,7 @@ module Mode : sig
 
   type 'name t =
     { name : 'name; args : bool list; loc : Loc.t }
-  [@@ deriving show]
+  [@@ deriving show, ord]
 
 end
 
@@ -145,7 +147,7 @@ module Program : sig
     (* Blocks *)
     | Begin of Loc.t
     | Namespace of Loc.t * Func.t
-    | Constraint of Loc.t * Func.t list
+    | Constraint of Loc.t * Func.t list * Func.t list
     | Shorten of Loc.t * (Func.t * Func.t) list (* prefix suffix *)
     | End of Loc.t
 
@@ -195,12 +197,17 @@ type program = {
   modes : Func.t Mode.t list;
   body : block list;
 }
+and block_constraint = {
+   clique : Func.t list;
+   ctx_filter : Func.t list;
+   rules : cattribute Chr.t list
+}
 and block =
   | Locals of Func.t list * program
   | Clauses of (Term.t,attribute) Clause.t list
   | Namespace of Func.t * program
   | Shorten of Func.t shorthand list * program
-  | Constraints of Func.t list * cattribute Chr.t list * program
+  | Constraints of block_constraint * program
 and attribute = {
   insertion : insertion option;
   id : string option;
@@ -220,6 +227,6 @@ and 'a shorthand = {
   full_name : 'a;
   short_name : 'a;
 }
-[@@deriving show]
+[@@deriving show, ord]
 
 end
