@@ -299,4 +299,59 @@ let () = declare "hdclause"
   ~description:"hdclause unification"
   ~typecheck:false
   ()
-  
\ No newline at end of file
+  
+let () = declare "oc_eta"
+  ~source_elpi:"oc_eta.elpi"
+  ~description:"eta expansion and occur check"
+  ~typecheck:true
+  ~expectation:Failure
+  ()
+
+let () = declare "eta_oc"
+  ~source_elpi:"eta_oc.elpi"
+  ~description:"eta expansion and occur check"
+  ~typecheck:true
+  ~expectation:Success
+  ()
+  
+let () = declare "bug_226"
+  ~source_elpi:"bug_226.elpi"
+  ~description:"move/unif bug"
+  ~typecheck:true
+  ~expectation:Success
+  ()
+
+let () = declare "chr-scope"
+  ~source_elpi:"chr-scope.elpi"
+  ~description:"chr-relocation"
+  ~typecheck:true
+  ~expectation:Success
+  ()
+
+let () = declare "chr-scope-change"
+  ~source_elpi:"chr-scope-change.elpi"
+  ~description:"chr-relocation"
+  ~typecheck:true
+  ~expectation:Success
+  ()
+
+let () = declare "chr-scope-change-err"
+  ~source_elpi:"chr-scope-change-failure.elpi"
+  ~description:"chr-relocation"
+  ~typecheck:true
+  ~expectation:(FailureOutput (Str.regexp "cannot be put in the desired context"))
+  ()
+
+let () = declare "chr_with_hypotheses"
+  ~source_elpi:"chr_with_hypotheses.elpi"
+  ~description:"chr_with_hypotheses"
+  ~typecheck:true
+  ~expectation:Success
+  ()
+
+let () = declare "bug-256"
+  ~source_elpi:"bug-256.elpi"
+  ~description:"move/unif"
+  ~typecheck:true
+  ~expectation:Success
+  ()
