@@ -284,6 +284,14 @@ X0 || X2 && X3 ==> X4
 |})))
   ()
 
+let () = declare "linear"
+  ~source_elpi:"linear.elpi"
+  ~description:"linear variable check"
+  ~typecheck:true
+  ~expectation:Test.(SuccessOutput (Str.regexp_string "Foo_Bar is linear"))
+  ()
+
+
 let () = declare "IO_COLON"
   ~source_elpi:"io_colon.elpi"
   ~description:"IO_COLON token"
@@ -303,6 +311,13 @@ let () = declare "graft_replace_err"
   ~expectation:Test.(FailureOutput (Str.regexp "name attribute"))
   ()
 
+let () = declare "mk_uv_meta"
+  ~source_elpi:"mk-evar-meta.elpi"
+  ~description:"uvar surgery at the meta level"
+  ~typecheck:true
+  ~expectation:Test.Success
+  ()
+
 let mk_tmp_file =
   let tmp = ref 0 in
   let dir = Filename.get_temp_dir_name () in
@@ -310,8 +325,9 @@ let mk_tmp_file =
     incr tmp;
     dir ^ Filename.dir_sep ^ x ^ "__" ^ string_of_int !tmp ^ "__" ^ y
     
-let sample = mk_tmp_file "trace.json" ".new"
-let () = declare "trace-browser"
+let () =
+  let sample = mk_tmp_file "trace.json" ".new" in
+  declare "trace-browser"
   ~source_elpi:"trace.elpi"
   ~description:"trace generation"
   ~typecheck:false
