@@ -99,3 +99,7 @@ let () = declare "dt_off"
   ~description:"(without) discrimination_tree indexing"
   ()
 
+  let () = declare "dt_bench"
+  ~source_elpi:"dt_bench.elpi"
+  ~description:"discrimination_tree on trees"
+  ()
